/*
 * HapticCmdDispatcher.c
 *
 *  Created on: Mar 21, 2014
 *      Author: a0220410
 */

#include "HapticCmdDispatcher.h"
#include "HapticErrNo.h"
#include "HapticsTarget.h"
#include "string.h"
#include "board.h"

static uint16_t g_ErrorNo = 0;
static uint8_t g_RegPage = 0;

static uint8_t HapticCommonBuf[HAPTIC_COMMON_BUF_SIZE];
static uint16_t g_CommBufPointer = 0;
static uint16_t g_TotalInCommBuf = 0;

uint8_t HapticCmdBuf[HAPTIC_CMD_BUF_SIZE];

void HapticCmdDispatch(uint16_t count)
{
	int rs = 0;
	uint16_t HapticCmdId = (uint16_t)(HapticCmdBuf[HAPTIC_REP_ID_OFFSET_H]<<8)|HapticCmdBuf[HAPTIC_REP_ID_OFFSET_L];
	g_ErrorNo = NO_ERROR;

	switch(HapticCmdId){
		case HAPTICS_REPORT_SINGLE_WRITE_OUT:
		{
			uint8_t SlaveAddr = HapticCmdBuf[HAPTIC_REP_ADDR_OFFSET];
			uint8_t RegPage = HapticCmdBuf[HAPTIC_REP_REGPAGE_OFFSET];
			uint8_t RegAddr = HapticCmdBuf[HAPTIC_REP_REGADDR_OFFSET];
			uint8_t Val = HapticCmdBuf[HAPTIC_REP_REGVAL_OFFSET];
			if(g_RegPage != RegPage){
				rs = StdI2C_TX_Single(SlaveAddr, 0xff, RegPage, &g_ErrorNo);
				if((rs == 1)&&(g_ErrorNo==NO_ERROR)){
					g_RegPage = RegPage;
				}
			}
			if((rs >= 0)&&(g_ErrorNo==NO_ERROR)){
				rs = StdI2C_TX_Single(SlaveAddr, RegAddr, Val, &g_ErrorNo);
			}
			HapticCmdBuf[HAPTIC_REP_ID_OFFSET_L] = (uint8_t)HAPTICS_REPORT_SINGLE_WRITE_IN;
			HapticCmdBuf[HAPTIC_REP_ID_OFFSET_H] = (uint8_t)(HAPTICS_REPORT_SINGLE_WRITE_IN>>8);
			HapticCmdBuf[count -3] = (uint8_t)g_ErrorNo;
			HapticCmdBuf[count -2] = (uint8_t)(g_ErrorNo>>8);
			if((rs == 1)&&(g_ErrorNo==NO_ERROR)){
				HapticCmdBuf[count -1] = 1;
			}else{
				HapticCmdBuf[count -1] = 0;
			}
		}
		break;

		case HAPTICS_REPORT_SINGLE_READ_OUT:
		{
			uint8_t SlaveAddr = HapticCmdBuf[HAPTIC_REP_ADDR_OFFSET];
			uint8_t RegPage = HapticCmdBuf[HAPTIC_REP_REGPAGE_OFFSET];
			uint8_t RegAddr = HapticCmdBuf[HAPTIC_REP_REGADDR_OFFSET];
			uint8_t RegVal = 0;
			if(g_RegPage != RegPage){
				rs = StdI2C_TX_Single(SlaveAddr, 0xff, RegPage, &g_ErrorNo);
				if((rs == 1)&&(g_ErrorNo==NO_ERROR)){
					g_RegPage = RegPage;
				}
			}
			if((rs >= 0)&&(g_ErrorNo==NO_ERROR)){
				rs = StdI2C_RX_Single(SlaveAddr, RegAddr, &RegVal, &g_ErrorNo);
			}
			HapticCmdBuf[HAPTIC_REP_ID_OFFSET_L] = (uint8_t)HAPTICS_REPORT_SINGLE_READ_IN;
			HapticCmdBuf[HAPTIC_REP_ID_OFFSET_H] = (uint8_t)(HAPTICS_REPORT_SINGLE_READ_IN>>8);
			HapticCmdBuf[count -3] = (uint8_t)g_ErrorNo;
			HapticCmdBuf[count -2] = (uint8_t)(g_ErrorNo>>8);
			if((rs == 1)&&(g_ErrorNo==NO_ERROR)){
				HapticCmdBuf[HAPTIC_REP_DATALEN_OFFSET] += 1;
				HapticCmdBuf[count -4] = RegVal;
				HapticCmdBuf[count -1] = 1;
			}else{
				HapticCmdBuf[count -1] = 0;
			}
		}
		break;

		case HAPTICS_REPORT_MULTI_WRITE_OUT:
		{
			uint8_t SlaveAddr = HapticCmdBuf[HAPTIC_REP_ADDR_OFFSET];
			uint8_t RegPage = HapticCmdBuf[HAPTIC_REP_REGPAGE_OFFSET];
			uint8_t RegAddr = HapticCmdBuf[HAPTIC_REP_REGADDR_OFFSET];
			uint8_t Count = HapticCmdBuf[HAPTIC_REP_DATALEN_OFFSET] - 4;
			if(g_RegPage != RegPage){
				rs = StdI2C_TX_Single(SlaveAddr, 0xff, RegPage, &g_ErrorNo);
				if((rs == 1)&&(g_ErrorNo==NO_ERROR)){
					g_RegPage = RegPage;
				}
			}
			if((rs >= 0)&&(g_ErrorNo==NO_ERROR)){
				rs = StdI2C_TX_Multiple(SlaveAddr, RegAddr, Count, &HapticCmdBuf[HAPTIC_REP_REGVAL_OFFSET], &g_ErrorNo);
			}
			HapticCmdBuf[HAPTIC_REP_ID_OFFSET_L] = (uint8_t)HAPTICS_REPORT_MULTI_WRITE_IN;
			HapticCmdBuf[HAPTIC_REP_ID_OFFSET_H] = (uint8_t)(HAPTICS_REPORT_MULTI_WRITE_IN>>8);
			HapticCmdBuf[count -3] = (uint8_t)g_ErrorNo;
			HapticCmdBuf[count -2] = (uint8_t)(g_ErrorNo>>8);
			if((rs == Count)&&(g_ErrorNo==NO_ERROR)){
				HapticCmdBuf[count -1] = Count;
			}else{
				HapticCmdBuf[count -1] = 0;
			}
		}
		break;

		case HAPTICS_REPORT_MULTI_READ_OUT:
		{
			uint8_t SlaveAddr = HapticCmdBuf[HAPTIC_REP_ADDR_OFFSET];
			uint8_t RegPage = HapticCmdBuf[HAPTIC_REP_REGPAGE_OFFSET];
			uint8_t RegAddr = HapticCmdBuf[HAPTIC_REP_REGADDR_OFFSET];
			uint8_t Count = HapticCmdBuf[HAPTIC_REP_REGCNT_OFFSET];
			if(g_RegPage != RegPage){
				rs = StdI2C_TX_Single(SlaveAddr, 0xff, RegPage, &g_ErrorNo);
				if((rs == 1)&&(g_ErrorNo==NO_ERROR)){
					g_RegPage = RegPage;
				}
			}
			if((rs >= 0)&&(g_ErrorNo==NO_ERROR)){
				if(Count == 1){
					rs = StdI2C_RX_Single(SlaveAddr, RegAddr, &HapticCmdBuf[HAPTIC_REP_REGCNT_OFFSET+1], &g_ErrorNo);
				}else{
					rs = StdI2C_RX_Multiple(SlaveAddr, RegAddr, Count, &HapticCmdBuf[HAPTIC_REP_REGCNT_OFFSET+1], &g_ErrorNo);
				}
			}
			HapticCmdBuf[HAPTIC_REP_ID_OFFSET_L] = (uint8_t)HAPTICS_REPORT_MULTI_READ_IN;
			HapticCmdBuf[HAPTIC_REP_ID_OFFSET_H] = (uint8_t)(HAPTICS_REPORT_MULTI_READ_IN>>8);
			HapticCmdBuf[count -3] = (uint8_t)g_ErrorNo;
			HapticCmdBuf[count -2] = (uint8_t)(g_ErrorNo>>8);
			if((rs == Count)&&(g_ErrorNo==NO_ERROR)){
				HapticCmdBuf[HAPTIC_REP_DATALEN_OFFSET] += Count;
				HapticCmdBuf[count -1] = Count;
			}else{
				HapticCmdBuf[count -1] = 0;
			}
		}
		break;

		case HAPTICS_REPORT_SINGLE_SETBIT_OUT:
		{
			uint8_t SlaveAddr = HapticCmdBuf[HAPTIC_REP_ADDR_OFFSET];
			uint8_t RegPage = HapticCmdBuf[HAPTIC_REP_REGPAGE_OFFSET];
			uint8_t RegAddr = HapticCmdBuf[HAPTIC_REP_REGADDR_OFFSET];
			uint8_t RegMsk = HapticCmdBuf[HAPTIC_REP_REGMSK_OFFSET];
			uint8_t RegBit = HapticCmdBuf[HAPTIC_REP_REGBIT_OFFSET];
			uint8_t RegVal = 0;

			if(g_RegPage != RegPage){
				rs = StdI2C_TX_Single(SlaveAddr, 0xff, RegPage, &g_ErrorNo);
				if((rs == 1)&&(g_ErrorNo==NO_ERROR)){
					g_RegPage = RegPage;
				}
			}
			if((rs >= 0)&&(g_ErrorNo==NO_ERROR)){
				rs = StdI2C_RX_Single(SlaveAddr, RegAddr, &RegVal, &g_ErrorNo);
				if((rs == 1)&&(g_ErrorNo==NO_ERROR)){
					if((RegVal & RegMsk) != (RegBit & RegMsk)){
						RegVal = (RegVal&~RegMsk)|(RegBit&RegMsk);
						rs = StdI2C_TX_Single(SlaveAddr, RegAddr, RegVal, &g_ErrorNo);
					}
				}
			}

			HapticCmdBuf[HAPTIC_REP_ID_OFFSET_L] = (uint8_t)HAPTICS_REPORT_SINGLE_SETBIT_IN;
			HapticCmdBuf[HAPTIC_REP_ID_OFFSET_H] = (uint8_t)(HAPTICS_REPORT_SINGLE_SETBIT_IN>>8);
			HapticCmdBuf[count -3] = (uint8_t)g_ErrorNo;
			HapticCmdBuf[count -2] = (uint8_t)(g_ErrorNo>>8);
			if((rs == 1)&&(g_ErrorNo==NO_ERROR)){
				HapticCmdBuf[count -1] = 1;
			}else{
				HapticCmdBuf[count -1] = 0;
			}
		}
		break;

		case HAPTICS_REPORT_MULTI_SETBIT_OUT:
		{
			uint8_t SlaveAddr = HapticCmdBuf[HAPTIC_REP_ADDR_OFFSET];
			uint8_t RegPage = HapticCmdBuf[HAPTIC_REP_REGPAGE_OFFSET];
			uint8_t RegNum = (HapticCmdBuf[HAPTIC_REP_DATALEN_OFFSET] - 3)/3;
			uint8_t RegAddr = 0;
			uint8_t RegMsk = 0;
			uint8_t RegBit = 0;
			uint8_t RegVal = 0;
			int i;

			if(g_RegPage != RegPage){
				rs = StdI2C_TX_Single(SlaveAddr, 0xff, RegPage, &g_ErrorNo);
				if((rs == 1)&&(g_ErrorNo==NO_ERROR)){
					g_RegPage = RegPage;
				}
			}
			if((rs >= 0)&&(g_ErrorNo==NO_ERROR)){
				for( i=0; i<RegNum; i++){
					RegAddr = HapticCmdBuf[HAPTIC_REP_REGPAGE_OFFSET + 1 + 3*i];
					RegMsk = HapticCmdBuf[HAPTIC_REP_REGPAGE_OFFSET + 2 + 3*i];
					RegBit = HapticCmdBuf[HAPTIC_REP_REGPAGE_OFFSET + 3 + 3*i];
					rs = StdI2C_RX_Single(SlaveAddr, RegAddr, &RegVal, &g_ErrorNo);
					if((rs == 1)&&(g_ErrorNo==NO_ERROR)){
						if((RegVal & RegMsk) != (RegBit & RegMsk)){
							RegVal = (RegVal&~RegMsk)|(RegBit&RegMsk);
							rs = StdI2C_TX_Single(SlaveAddr, RegAddr, RegVal, &g_ErrorNo);
							if((rs != 1)||(g_ErrorNo!=NO_ERROR)){
								break;
							}
						}
					}else{
						break;
					}
				}
			}

			HapticCmdBuf[HAPTIC_REP_ID_OFFSET_L] = (uint8_t)HAPTICS_REPORT_MULTI_SETBIT_IN;
			HapticCmdBuf[HAPTIC_REP_ID_OFFSET_H] = (uint8_t)(HAPTICS_REPORT_MULTI_SETBIT_IN>>8);
			HapticCmdBuf[count -3] = (uint8_t)g_ErrorNo;
			HapticCmdBuf[count -2] = (uint8_t)(g_ErrorNo>>8);
			if((rs == 1)&&(g_ErrorNo==NO_ERROR)){
				HapticCmdBuf[count -1] = RegNum;
			}else{
				HapticCmdBuf[count -1] = 0;
			}
		}
		break;

		case HAPTICS_REPORT_TARGET_DETECT_OUT:
		{
			uint8_t RegVal = 0;
			uint8_t target = 0;
			uint8_t StrLen = 0;

			rs = StdI2C_RX_Single(IICADDR_DRV260X, DEVICEID_REG_DRV260X, &RegVal, &g_ErrorNo);
			if((rs == 1)&&(g_ErrorNo == NO_ERROR)){
				target = (RegVal&DEVICEID_MSK_DRV260X)>>DEVICEID_SFT_DRV260X;
				if(target == DRV2605){
					StrLen = strlen(TARGET_DRV2605);
					memcpy(&HapticCmdBuf[HAPTIC_REP_PORT_OFFSET+1], TARGET_DRV2605, StrLen);
				}else if(target == DRV2605L){
					StrLen = strlen(TARGET_DRV2605L);
					memcpy(&HapticCmdBuf[HAPTIC_REP_PORT_OFFSET+1], TARGET_DRV2605L, StrLen);
				}else if(target == DRV2604){
					StrLen = strlen(TARGET_DRV2604);
					memcpy(&HapticCmdBuf[HAPTIC_REP_PORT_OFFSET+1], TARGET_DRV2604, StrLen);
				}else if(target == DRV2604L){
					StrLen = strlen(TARGET_DRV2604L);
					memcpy(&HapticCmdBuf[HAPTIC_REP_PORT_OFFSET+1], TARGET_DRV2604L, StrLen);
				}else{
				    target =  (RegVal&DEVICEID_MSK_DRV262X)>>DEVICEID_SFT_DRV262X;
				    if(target == DRV2624){
                        StrLen = strlen(TARGET_DRV2624mini);
                        memcpy(&HapticCmdBuf[HAPTIC_REP_PORT_OFFSET+1], TARGET_DRV2624mini, StrLen);
				    }else if(target == DRV2625){
                        StrLen = strlen(TARGET_DRV2625mini);
                        memcpy(&HapticCmdBuf[HAPTIC_REP_PORT_OFFSET+1], TARGET_DRV2625mini, StrLen);
				    }else{
				        StrLen = strlen(TARGET_DRVxxxx);
				        memcpy(&HapticCmdBuf[HAPTIC_REP_PORT_OFFSET+1], TARGET_DRVxxxx, StrLen);
				    }
				}
			}else if(g_ErrorNo == I2C_NO_ACK){
				/* detect target with multiple register pages */
				rs = StdI2C_TX_Single(IICADDR_DRV2667, 0xff, 0, &g_ErrorNo);
				if((rs == 1)&&(g_ErrorNo == NO_ERROR)){
					g_RegPage = 0;
#if 0
					rs = StdI2C_RX_Single(IICADDR_DRV2667, DEVICEID_REG_DRV2667, &RegVal, &g_ErrorNo);
					if((rs == 1)&&(g_ErrorNo == NO_ERROR)){
						target = (RegVal&DEVICEID_MSK_DRV2667)>>DEVICEID_SFT_DRV2667;
						if(target == DRV2667){
							StrLen = strlen(TARGET_DRV2667);
							memcpy(&HapticCmdBuf[HAPTIC_REP_PORT_OFFSET+1], TARGET_DRV2667, StrLen);
						}else{
							StrLen = strlen(TARGET_DRVxxxx);
							memcpy(&HapticCmdBuf[HAPTIC_REP_PORT_OFFSET+1], TARGET_DRVxxxx, StrLen);
						}
					}
#else
					rs = StdI2C_RX_Single(IICADDR_DRV2667, 0x01, &RegVal, &g_ErrorNo);
					if((rs == 1)&&(g_ErrorNo==NO_ERROR)){
						if((RegVal & 0x78) != 0x38){
							RegVal = (RegVal&~0x78)|0x38;
							rs = StdI2C_TX_Single(IICADDR_DRV2667, 0x01, RegVal, &g_ErrorNo);
						}
					}

					StrLen = strlen(TARGET_DRV2667);
					memcpy(&HapticCmdBuf[HAPTIC_REP_PORT_OFFSET+1], TARGET_DRV2667, StrLen);
#endif
				}
			}

			HapticCmdBuf[HAPTIC_REP_ID_OFFSET_L] = (uint8_t)HAPTICS_REPORT_TARGET_DETECT_IN;
			HapticCmdBuf[HAPTIC_REP_ID_OFFSET_H] = (uint8_t)(HAPTICS_REPORT_TARGET_DETECT_IN>>8);
			HapticCmdBuf[count -3] = (uint8_t)g_ErrorNo;
			HapticCmdBuf[count -2] = (uint8_t)(g_ErrorNo>>8);
			if((StrLen > 0)&&(g_ErrorNo==NO_ERROR)){
				HapticCmdBuf[HAPTIC_REP_DATALEN_OFFSET] += StrLen;
				HapticCmdBuf[count -1] = StrLen;
			}else{
				HapticCmdBuf[count -1] = 0;
			}
		}
		break;

		case HAPTICS_REPORT_FIFO_PLAYBACK_OUT:
		{
			uint8_t packet = HapticCmdBuf[HAPTIC_REP_FIFO_PACKETID_OFFSET];
			uint8_t datacount = HapticCmdBuf[HAPTIC_REP_DATALEN_OFFSET];
			uint8_t totalcount = HapticCmdBuf[HAPTIC_REP_FIFO_DATALEN_LSB_OFFSET];
			uint8_t slaveAddr = HapticCmdBuf[HAPTIC_REP_FIFO_ADDR_OFFSET];

			if((totalcount <= DRV2667_FIFO_LEN)&&(datacount>6)){
				if(packet == 0){
					g_CommBufPointer = 0;
					g_TotalInCommBuf = 0;
				}
				datacount -= 6; 
				memcpy(&HapticCommonBuf[g_CommBufPointer], &HapticCmdBuf[HAPTIC_REP_FIFO_DATA_OFFSET], datacount);
				g_CommBufPointer += datacount;
				g_TotalInCommBuf += datacount;

				if(g_TotalInCommBuf == totalcount){
					rs = StdI2C_TX_Single(slaveAddr, 0xff, 0, &g_ErrorNo);
					if((rs == 1)&&(g_ErrorNo == NO_ERROR)){
						rs = StdI2C_TX_Multiple(slaveAddr, DRV2667_FIFO_REG, totalcount, HapticCommonBuf, &g_ErrorNo);
					}
				}
			}else{
				g_ErrorNo = INVALID_PARAMETER;
			}

			HapticCmdBuf[HAPTIC_REP_ID_OFFSET_L] = (uint8_t)HAPTICS_REPORT_FIFO_PLAYBACK_IN;
			HapticCmdBuf[HAPTIC_REP_ID_OFFSET_H] = (uint8_t)(HAPTICS_REPORT_FIFO_PLAYBACK_IN>>8);
			HapticCmdBuf[count -3] = (uint8_t)g_ErrorNo;
			HapticCmdBuf[count -2] = (uint8_t)(g_ErrorNo>>8);
			if((rs >= 0)&&(g_ErrorNo==NO_ERROR))
				HapticCmdBuf[count -1] = datacount;
			else
				HapticCmdBuf[count -1] = 0;
		}
		break;

		case HAPTICS_REPORT_GPIO_CONFIG_OUT:
		{
			uint8_t Config = HapticCmdBuf[HAPTIC_REP_GPIO_CONFIG_OFFSET];
			uint8_t BankCount = HapticCmdBuf[HAPTIC_REP_GPIO_BANKCOUNT_OFFSET];
			uint8_t Bank = 0, pin_LSB = 0,pin_MSB = 0, ic = 0;
			uint16_t pin = 0;

			for(ic=0;ic<BankCount;ic++)
			{
				Bank = HapticCmdBuf[HAPTIC_REP_GPIO_BANK_OFFSET + ic*3];
				pin_LSB = HapticCmdBuf[HAPTIC_REP_GPIO_PINNUM_OFFSET + ic*3];
				pin_MSB = HapticCmdBuf[HAPTIC_REP_GPIO_PINNUM_OFFSET + 1 + ic*3];
				pin = ((uint16_t)(pin_MSB<<8)|pin_LSB);
				rs = GPIO_Config(Config, Bank, pin, &g_ErrorNo);
				if((rs < 0)||(g_ErrorNo!=NO_ERROR)){
					break;
				}
			}

			HapticCmdBuf[HAPTIC_REP_ID_OFFSET_L] = (uint8_t)HAPTICS_REPORT_GPIO_CONFIG_IN;
			HapticCmdBuf[HAPTIC_REP_ID_OFFSET_H] = (uint8_t)(HAPTICS_REPORT_GPIO_CONFIG_IN>>8);
			HapticCmdBuf[count -3] = (uint8_t)g_ErrorNo;
			HapticCmdBuf[count -2] = (uint8_t)(g_ErrorNo>>8);
			if((rs>=0)&&(g_ErrorNo==NO_ERROR))
				HapticCmdBuf[count -1] = BankCount;
			else
				HapticCmdBuf[count -1] = 0;
		}
		break;
		
		case HAPTICS_REPORT_ECHO_OUT:
		{
			HapticCmdBuf[HAPTIC_REP_ID_OFFSET_L] = (uint8_t)HAPTICS_REPORT_ECHO_IN;
			HapticCmdBuf[HAPTIC_REP_ID_OFFSET_H] = (uint8_t)(HAPTICS_REPORT_ECHO_IN>>8);
			HapticCmdBuf[count -3] = 0;
			HapticCmdBuf[count -2] = 0;
			HapticCmdBuf[count -1] = HapticCmdBuf[HAPTIC_REP_DATALEN_OFFSET];
		}
		break;

		case HAPTICS_REPORT_RAW_WRITE_OUT:
		{
			uint8_t datalen = HapticCmdBuf[HAPTIC_REP_DATALEN_OFFSET] - 4;
			uint8_t packetcount = HapticCmdBuf[HAPTIC_REP_PACKET_COUNT_OFFSET];
			uint8_t packetid = HapticCmdBuf[HAPTIC_REP_PACKET_ID_OFFSET];
			uint8_t slaveAddr = HapticCmdBuf[HAPTIC_REP_ADDR_OFFSET];

			if((packetid == 0) && (datalen == 1)){
				rs = RawI2C_TX_Single(slaveAddr, HapticCmdBuf[HAPTIC_REP_PACKET_DATA_OFFSET],  &g_ErrorNo);
			}else{
				if(packetid == 0)	g_CommBufPointer = 0;

				if((g_CommBufPointer + datalen) <= HAPTIC_COMMON_BUF_SIZE){
					memcpy(&HapticCommonBuf[g_CommBufPointer], &HapticCmdBuf[HAPTIC_REP_PACKET_DATA_OFFSET], datalen);
					g_CommBufPointer += datalen;

					if(packetid < (packetcount-1)){
						//pass
					}else if(packetid == (packetcount-1)){
						rs = RawI2C_TX_Multiple(slaveAddr, g_CommBufPointer, HapticCommonBuf, &g_ErrorNo);
					}else{
						g_ErrorNo = INVALID_PARAMETER;
						rs = -1;
					}
				}else{
					g_ErrorNo = BUFFER_OVERFLOW;
					rs = -1;
				}
			}

			HapticCmdBuf[HAPTIC_REP_ID_OFFSET_L] = (uint8_t)HAPTICS_REPORT_RAW_WRITE_IN;
			HapticCmdBuf[HAPTIC_REP_ID_OFFSET_H] = (uint8_t)(HAPTICS_REPORT_RAW_WRITE_IN>>8);
			HapticCmdBuf[count -3] = (uint8_t)g_ErrorNo;
			HapticCmdBuf[count -2] = (uint8_t)(g_ErrorNo>>8);
			if((rs>=0)&&(g_ErrorNo==NO_ERROR))
				HapticCmdBuf[count -1] = datalen;
			else
				HapticCmdBuf[count -1] = 0;
		}
		break;

		case HAPTICS_REPORT_RAW_READ_OUT:
		{
			uint8_t readcount = HapticCmdBuf[HAPTIC_REP_DATA_COUNT_OFFSET];
			uint8_t packetid = HapticCmdBuf[HAPTIC_REP_PACKET_ID_OFFSET];
			uint8_t datainPacket = 0;

			if(packetid == 0){
				uint8_t slaveAddr = HapticCmdBuf[HAPTIC_REP_ADDR_OFFSET];
				g_CommBufPointer = 0;
				if(readcount == 0){
					g_ErrorNo = INVALID_PARAMETER;
					rs = -1;
				}else if(readcount == 1){
					rs = RawI2C_RX_Single(slaveAddr, &HapticCmdBuf[HAPTIC_REP_PACKET_DATA_OFFSET], &g_ErrorNo);
					if((rs == 1)&&(g_ErrorNo == NO_ERROR))  datainPacket = rs;
				}else{
					g_TotalInCommBuf = 0;
					rs = RawI2C_RX_Multiple(slaveAddr, readcount, HapticCommonBuf, &g_ErrorNo);
					if((rs == readcount) && (g_ErrorNo == NO_ERROR)){
						g_TotalInCommBuf = rs;
						if(rs <= HAPTIC_REP_RAW_READ_DATALEN){
							memcpy(&HapticCmdBuf[HAPTIC_REP_PACKET_DATA_OFFSET], HapticCommonBuf, rs);
							datainPacket = rs;
							g_TotalInCommBuf = 0;
						}
						else
						{
							memcpy(&HapticCmdBuf[HAPTIC_REP_PACKET_DATA_OFFSET], HapticCommonBuf, HAPTIC_REP_RAW_READ_DATALEN);
							datainPacket = HAPTIC_REP_RAW_READ_DATALEN;
							g_CommBufPointer = HAPTIC_REP_RAW_READ_DATALEN;
							g_TotalInCommBuf -= datainPacket;
						}
					}
				}
			}else{
				if(g_TotalInCommBuf>0){
					if(readcount <= HAPTIC_REP_RAW_READ_DATALEN){
						if(readcount <= g_TotalInCommBuf){
							memcpy(&HapticCmdBuf[HAPTIC_REP_PACKET_DATA_OFFSET], &HapticCommonBuf[g_CommBufPointer], readcount);
							datainPacket = readcount;
							g_CommBufPointer += readcount;
							g_TotalInCommBuf -= readcount;
						}else{
							memcpy(&HapticCmdBuf[HAPTIC_REP_PACKET_DATA_OFFSET], &HapticCommonBuf[g_CommBufPointer], g_TotalInCommBuf);
							datainPacket = g_TotalInCommBuf;
							g_TotalInCommBuf = 0;
						}
					}else{
						if(g_TotalInCommBuf >= HAPTIC_REP_RAW_READ_DATALEN){
							memcpy(&HapticCmdBuf[HAPTIC_REP_PACKET_DATA_OFFSET], &HapticCommonBuf[g_CommBufPointer], HAPTIC_REP_RAW_READ_DATALEN);
							datainPacket = HAPTIC_REP_RAW_READ_DATALEN;
							g_TotalInCommBuf -= HAPTIC_REP_RAW_READ_DATALEN;
							g_CommBufPointer += HAPTIC_REP_RAW_READ_DATALEN;
						}else{
							memcpy(&HapticCmdBuf[HAPTIC_REP_PACKET_DATA_OFFSET], &HapticCommonBuf[g_CommBufPointer], g_TotalInCommBuf);
							datainPacket = g_TotalInCommBuf;
							g_TotalInCommBuf = 0;
						}
					}
				}else{
					g_ErrorNo = I2C_NO_DATA;
				}
			}

			HapticCmdBuf[HAPTIC_REP_ID_OFFSET_L] = (uint8_t)HAPTICS_REPORT_RAW_READ_OUT;
			HapticCmdBuf[HAPTIC_REP_ID_OFFSET_H] = (uint8_t)(HAPTICS_REPORT_RAW_READ_IN>>8);
			HapticCmdBuf[HAPTIC_REP_DATALEN_OFFSET] += datainPacket;
			HapticCmdBuf[count -3] = (uint8_t)g_ErrorNo;
			HapticCmdBuf[count -2] = (uint8_t)(g_ErrorNo>>8);
			if((rs >= 0)&&(g_ErrorNo == NO_ERROR))
				HapticCmdBuf[count -1] = datainPacket;
			else
				HapticCmdBuf[count -1] = 0;
		}
		break;

		case HAPTICS_REPORT_PWM_OUTPUT_OUT:
		{
			uint8_t period_lsb = HapticCmdBuf[HAPTIC_REP_PERIOD_OFFSET];
			uint8_t period_msb = HapticCmdBuf[HAPTIC_REP_PERIOD_OFFSET+1];
			uint8_t duty_lsb = HapticCmdBuf[HAPTIC_REP_DUTY_OFFSET];
			uint8_t duty_msb = HapticCmdBuf[HAPTIC_REP_DUTY_OFFSET+1];
			uint8_t sw = HapticCmdBuf[HAPTIC_REP_SW_OFFSET];
			uint16_t freq = ((uint16_t)period_msb<<8)|period_lsb;
			uint16_t duty = ((uint16_t)duty_msb<<8)|duty_lsb;

			rs = DualPWMConfig(freq, duty, sw, &g_ErrorNo);

			HapticCmdBuf[HAPTIC_REP_ID_OFFSET_L] = (uint8_t)HAPTICS_REPORT_PWM_OUTPUT_IN;
			HapticCmdBuf[HAPTIC_REP_ID_OFFSET_H] = (uint8_t)(HAPTICS_REPORT_PWM_OUTPUT_IN>>8);
			HapticCmdBuf[count -3] = (uint8_t)g_ErrorNo;
			HapticCmdBuf[count -2] = (uint8_t)(g_ErrorNo>>8);
			if((rs >= 0)&&(g_ErrorNo == NO_ERROR))
				HapticCmdBuf[count -1] = 1;
			else
				HapticCmdBuf[count -1] = 0;
		}
		break;

		default:
		{
			g_ErrorNo = UNKNOWN_REPORT;
			HapticCmdBuf[count -3] = (uint8_t)g_ErrorNo;
			HapticCmdBuf[count -2] = (uint8_t)(g_ErrorNo>>8);
			HapticCmdBuf[count -1] = 0;
		}
		break;
	}
}

