/*
 * HapticCmdDispatcher.h
 *
 *  Created on: Mar 21, 2014
 *      Author: a0220410
 */

#ifndef DRV_USBCOM_EVM_HAPTICCMDDISPATCHER_H_
#define DRV_USBCOM_EVM_HAPTICCMDDISPATCHER_H_

#include <stdint.h>

/* common macro */
#define MSP430_HIDREPORT_LEN		2
#define HAPTIC_REP_ID_OFFSET_L		(2 - MSP430_HIDREPORT_LEN)
#define HAPTIC_REP_ID_OFFSET_H		(3 - MSP430_HIDREPORT_LEN)
#define HAPTIC_REP_DATALEN_OFFSET	(4 - MSP430_HIDREPORT_LEN)
#define HAPTIC_REP_PORT_OFFSET		(5 - MSP430_HIDREPORT_LEN)

/* I2C Write Macro */
#define HAPTIC_REP_ADDR_OFFSET		(6 - MSP430_HIDREPORT_LEN)
#define HAPTIC_REP_REGPAGE_OFFSET	(7 - MSP430_HIDREPORT_LEN)
#define HAPTIC_REP_REGADDR_OFFSET	(8 - MSP430_HIDREPORT_LEN)
#define HAPTIC_REP_REGVAL_OFFSET	(9 - MSP430_HIDREPORT_LEN)
/* I2C Multiple Read */
#define HAPTIC_REP_REGCNT_OFFSET	(9 - MSP430_HIDREPORT_LEN)
/* I2C setbit */
#define HAPTIC_REP_REGMSK_OFFSET	(9 - MSP430_HIDREPORT_LEN)
#define HAPTIC_REP_REGBIT_OFFSET	(10 - MSP430_HIDREPORT_LEN)

#define HAPTIC_REP_FIFO_REPEAT_OFFSET		(5 - MSP430_HIDREPORT_LEN)
#define HAPTIC_REP_FIFO_PORT_OFFSET			(6 - MSP430_HIDREPORT_LEN)
#define HAPTIC_REP_FIFO_ADDR_OFFSET			(7 - MSP430_HIDREPORT_LEN)
#define HAPTIC_REP_FIFO_DATALEN_LSB_OFFSET	(8 - MSP430_HIDREPORT_LEN)
#define HAPTIC_REP_FIFO_DATALEN_MSB_OFFSET	(9 - MSP430_HIDREPORT_LEN)
#define HAPTIC_REP_FIFO_PACKETID_OFFSET		(10 - MSP430_HIDREPORT_LEN)
#define HAPTIC_REP_FIFO_DATA_OFFSET			(11 - MSP430_HIDREPORT_LEN)
#define HAPTIC_REP_FIFO_DATALEN 100
/* GPIO config */
#define HAPTIC_REP_GPIO_CONFIG_OFFSET		(5 - MSP430_HIDREPORT_LEN)
#define HAPTIC_REP_GPIO_BANKCOUNT_OFFSET	(6 - MSP430_HIDREPORT_LEN)
#define HAPTIC_REP_GPIO_BANK_OFFSET			(7 - MSP430_HIDREPORT_LEN)
#define HAPTIC_REP_GPIO_PINNUM_OFFSET_L		(8 - MSP430_HIDREPORT_LEN)
#define HAPTIC_REP_GPIO_PINNUM_OFFSET_H		(9 - MSP430_HIDREPORT_LEN)
#define HAPTIC_REP_GPIO_PINNUM_OFFSET		(8 - MSP430_HIDREPORT_LEN)

/* I2C RAW Write */
#define HAPTIC_REP_PACKET_COUNT_OFFSET		(7 - MSP430_HIDREPORT_LEN)
#define HAPTIC_REP_PACKET_ID_OFFSET			(8 - MSP430_HIDREPORT_LEN)
#define HAPTIC_REP_PACKET_DATA_OFFSET		(9 - MSP430_HIDREPORT_LEN)

/* I2C RAW Read */
#define HAPTIC_REP_DATA_COUNT_OFFSET		(7 - MSP430_HIDREPORT_LEN)
#define HAPTIC_REP_PACKET_ID_OFFSET			(8 - MSP430_HIDREPORT_LEN)
#define HAPTIC_REP_RAW_READ_DATALEN			52

/* PWM output */
#define HAPTIC_REP_PERIOD_OFFSET			(5 - MSP430_HIDREPORT_LEN)
#define HAPTIC_REP_DUTY_OFFSET				(7 - MSP430_HIDREPORT_LEN)
#define HAPTIC_REP_SW_OFFSET				(9 - MSP430_HIDREPORT_LEN)

#define HAPTIC_CMD_BUF_SIZE 64
#define HAPTIC_COMMON_BUF_SIZE (255+2)

typedef enum HapticsReportID_t {
	HAPTICS_REPORT_SINGLE_WRITE_OUT     = 0x0001,
	HAPTICS_REPORT_SINGLE_WRITE_IN      = 0x0002,
	HAPTICS_REPORT_SINGLE_READ_OUT      = 0x0003,
	HAPTICS_REPORT_SINGLE_READ_IN       = 0x0004,
	HAPTICS_REPORT_MULTI_WRITE_OUT      = 0x0005,
	HAPTICS_REPORT_MULTI_WRITE_IN       = 0x0006,
	HAPTICS_REPORT_MULTI_READ_OUT       = 0x0007,
	HAPTICS_REPORT_MULTI_READ_IN        = 0x0008,
	HAPTICS_REPORT_SINGLE_SETBIT_OUT    = 0x0009,
	HAPTICS_REPORT_SINGLE_SETBIT_IN     = 0x000a,
	HAPTICS_REPORT_MULTI_SETBIT_OUT     = 0x000b,
	HAPTICS_REPORT_MULTI_SETBIT_IN      = 0x000c,
	HAPTICS_REPORT_TARGET_DETECT_OUT    = 0x000d,
	HAPTICS_REPORT_TARGET_DETECT_IN     = 0x000e,
	HAPTICS_REPORT_GPIO_CONFIG_OUT		= 0x000f,
	HAPTICS_REPORT_GPIO_CONFIG_IN		= 0x0010,
	HAPTICS_REPORT_FIFO_PLAYBACK_OUT 	= 0x0011,
	HAPTICS_REPORT_FIFO_PLAYBACK_IN 	= 0x0012,
	HAPTICS_REPORT_ECHO_OUT 			= 0x0013,
	HAPTICS_REPORT_ECHO_IN 				= 0x0014,
	HAPTICS_REPORT_RAW_WRITE_OUT		= 0x0015,
	HAPTICS_REPORT_RAW_WRITE_IN			= 0x0016,
	HAPTICS_REPORT_RAW_READ_OUT			= 0x0017,
	HAPTICS_REPORT_RAW_READ_IN			= 0x0018,
	HAPTICS_REPORT_PWM_OUTPUT_OUT		= 0x0019,
	HAPTICS_REPORT_PWM_OUTPUT_IN		= 0x001a
}HapticsReportID;

extern uint8_t HapticCmdBuf[];

void HapticCmdDispatch(uint16_t count);
#endif /* DRV_USBCOM_EVM_HAPTICCMDDISPATCHER_H_ */
