/*
 * board_timer.c
 *
 *  Created on: Apr 1, 2014
 *      Author: a0220410
 */
#include "driverlib.h"
#include "timer_a.h"
#include "HapticErrNo.h"
#include "board_timer.h"
#include "board_config.h"

static uint8_t g_isPWMOn = 0;
static uint16_t g_PWMPeriod = 0;
static uint8_t g_isHighFreq = 0;

void board_timer_init(){

	GPIO_setOutputLowOnPin(GPIO_PORT_P1,GPIO_PIN2 + GPIO_PIN3);

	/* init periodial time for measurement */
	//Start timer in continuous mode sourced by SMCLK
	TIMER_A_configureContinuousMode( PERIODICAL_TIMER,
									 TIMER_A_CLOCKSOURCE_SMCLK,
									 TIMER_A_CLOCKSOURCE_DIVIDER_64,//TIMER_A_CLOCKSOURCE_DIVIDER_1,
									 TIMER_A_TAIE_INTERRUPT_DISABLE,
									 TIMER_A_DO_CLEAR
									 );

	//Initiaze compare mode
	TIMER_A_clearCaptureCompareInterruptFlag(PERIODICAL_TIMER,
											 TIMER_A_CAPTURECOMPARE_REGISTER_0
											 );
	TIMER_A_initCompare(PERIODICAL_TIMER,
						TIMER_A_CAPTURECOMPARE_REGISTER_0,
						TIMER_A_CAPTURECOMPARE_INTERRUPT_ENABLE,
						TIMER_A_OUTPUTMODE_OUTBITVALUE,
						COMPARE_VALUE
						);

	/* init T2 timer for FIFO communication and control */
		//Start timer2 in continuous mode sourced by SMCLK

		TIMER_A_configureContinuousMode( FIFOCOUNT_TIMER,
										 TIMER_A_CLOCKSOURCE_SMCLK,
										 TIMER_A_CLOCKSOURCE_DIVIDER_64,//TIMER_A_CLOCKSOURCE_DIVIDER_1,
										 TIMER_A_TAIE_INTERRUPT_DISABLE,
										 TIMER_A_DO_CLEAR
										 );

		//Initiaze compare mode
		TIMER_A_clearCaptureCompareInterruptFlag(FIFOCOUNT_TIMER,
												 TIMER_A_CAPTURECOMPARE_REGISTER_0
												 );
		TIMER_A_initCompare(FIFOCOUNT_TIMER,
							TIMER_A_CAPTURECOMPARE_REGISTER_0,
							TIMER_A_CAPTURECOMPARE_INTERRUPT_ENABLE,
							TIMER_A_OUTPUTMODE_OUTBITVALUE,
							COMPARE_VALUE
							);

}

static void DualPWM_Start(uint8_t clkDivider,uint16_t timerPeriod, uint16_t dutyCycle)
{
	GPIO_setAsPeripheralModuleFunctionOutputPin(GPIO_PORT_P1,GPIO_PIN2 + GPIO_PIN3);

	g_PWMPeriod = timerPeriod;
	TIMER_A_configureUpDownMode(PWM_TIMER,
                            TIMER_A_CLOCKSOURCE_SMCLK,
                            clkDivider,
                            timerPeriod,
                            TIMER_A_TAIE_INTERRUPT_DISABLE,
                            TIMER_A_CCIE_CCR0_INTERRUPT_DISABLE,
                            TIMER_A_DO_CLEAR
                            );

    TIMER_A_startCounter(PWM_TIMER,TIMER_A_UPDOWN_MODE);

    //Initialize compare mode to generate PWM1
    TIMER_A_initCompare(PWM_TIMER,
    					PWM_CCR_1P2,
                        TIMER_A_CAPTURECOMPARE_INTERRUPT_DISABLE,
                        TIMER_A_OUTPUTMODE_TOGGLE_RESET,
                        dutyCycle
                        );

    //Initialize compare mode to generate PWM2
    TIMER_A_initCompare(PWM_TIMER,
    					PWM_CCR_1P3,
    					TIMER_A_CAPTURECOMPARE_INTERRUPT_DISABLE,
    					TIMER_A_OUTPUTMODE_TOGGLE_SET,
    					timerPeriod - dutyCycle
                        );

}

static void PWM_SetDutyCycle(uint16_t period, uint16_t dutyCycle)
{
	TIMER_A_setCompareValue(PWM_TIMER, PWM_CCR_1P2, dutyCycle);
	TIMER_A_setCompareValue(PWM_TIMER, PWM_CCR_1P3, period - dutyCycle);
}

static void PWM_SetPeriod(uint16_t period)
{
	if(g_PWMPeriod != period){
		TIMER_A_setCounterValue(PWM_TIMER, period);
		g_PWMPeriod = period;
	}
}

static void PWM_Stop()
{
	g_PWMPeriod = 0;
	TIMER_A_stop(PWM_TIMER);
	GPIO_setAsOutputPin(GPIO_PORT_P1,GPIO_PIN2 + GPIO_PIN3);
	GPIO_setOutputLowOnPin(GPIO_PORT_P1,GPIO_PIN2 + GPIO_PIN3);
}

int DualPWMConfig(uint32_t freq, uint16_t duty, uint8_t sw, uint16_t *pErrNo)
{
	int ret = 0;
	uint16_t period = 0;
	uint16_t dutycycle = 0;
	uint8_t isHighFreq = 0;
	uint8_t clkDivider = 0;

	if(freq > PWM_FREQ_THRESHOLD){
		clkDivider = PWM_CLOCKSOURCE_DIVIDER_H;
		period = PWM_NUMOFCOUNTER_H/freq;
		isHighFreq = 1;
	}else{
		clkDivider = PWM_CLOCKSOURCE_DIVIDER_L;
		period = PWM_NUMOFCOUNTER_L/freq;
		isHighFreq = 0;
	}

	dutycycle = ((uint32_t)period*duty)/100;

	if((dutycycle < period)&&(sw==1)&&((g_isPWMOn == 0)||(isHighFreq != g_isHighFreq))){
		//turn on PWM
		DualPWM_Start(clkDivider, period, dutycycle);
		g_isHighFreq = isHighFreq;
		g_isPWMOn = 1;
	}else if((sw==0)&&(g_isPWMOn == 1)){
		//switch off PWM
		PWM_Stop();
		g_isPWMOn = 0;
	}else if((sw==1)&&(g_isPWMOn == 1)&&(dutycycle < period)){
		//update duty cycle
		PWM_SetPeriod(period);
		PWM_SetDutyCycle(period, dutycycle);
	}else if((sw==0)&&(g_isPWMOn == 0)){
		//pass
	}else{
		ret = -1;
	}

	if(ret == 0) *pErrNo = NO_ERROR;
	else *pErrNo = INVALID_PARAMETER;

	return ret;
}

void Periodical_Work_Start(){
	TIMER_A_startCounter( PERIODICAL_TIMER,
						  TIMER_A_CONTINUOUS_MODE
						  );
}
//Timer for FIFO operation
void FIFO_Work_Start(){
	TIMER_A_startCounter( FIFOCOUNT_TIMER,
						  TIMER_A_CONTINUOUS_MODE
						  );
}

void Periodical_Work_Stop(){
	TIMER_A_stop(PERIODICAL_TIMER);
}

void FIFO_Work_Stop(){
	TIMER_A_stop(FIFOCOUNT_TIMER);
}
//******************************************************************************
//
//This is the TIMER1_A3 interrupt vector service routine.
//
//******************************************************************************

#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector=TIMER1_A0_VECTOR
__interrupt
#elif defined(__GNUC__)
__attribute__((interrupt(TIMER1_A0_VECTOR)))
#endif
void TIMER1_A0_ISR(void)

{
        uint16_t compVal = TIMER_A_getCaptureCompareCount(TIMER_A1_BASE,
                                                          TIMER_A_CAPTURECOMPARE_REGISTER_0)
                           + COMPARE_VALUE;

        //Toggle P1.0
        GPIO_toggleOutputOnPin(
        		LED_PORT_M,
        		LED_PIN_M1
                );

        //Add Offset to CCR0
        TIMER_A_setCompareValue(TIMER_A1_BASE,
                                TIMER_A_CAPTURECOMPARE_REGISTER_0,
                                compVal
                                );
}


//******************************************************************************
//
//This is the TIMER2_A3 interrupt vector service routine.
//By Kevin Zhang
//
//******************************************************************************

#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector=TIMER2_A0_VECTOR
__interrupt
#elif defined(__GNUC__)
__attribute__((interrupt(TIMER2_A0_VECTOR)))
#endif
void TIMER2_A0_ISR(void)
//use of TIMER2
{
	FIFO_Work_Stop();
#if 0
	while(writefifocount < FIFODatalen)
	{
		rs = FIFOI2C_TX_Multiple(SlaveAddr, 0x0B, FIFOCOUNT, &fifoaddrtemp[ncount], &g_ErrorNo);
		if(rs == FIFOCOUNT)
		{
			writefifocount += rs;
			ncount += rs;
		}
		else
		{
			writefifocount += rs-2;
			ncount += rs-2;
		}
	}
#endif
}

